from typing import List, Optional, Any, Dict

from pydantic import BaseModel, Field

class FinancialReportQuery(BaseModel):
    pageNo: int = Field(..., ge=1, le=1000, description="页码，必须大于等于1且小于等于1000")
    pageSize: int = Field(..., ge=1, le=50, description="每页条数，必须大于等于1且小于等于50")

    # 可选参数
    year: Optional[int] = Field(None, ge=1, le=2100, description="时间选择，必须为正整数或特定年份且小于等于2100，或留空")
    stockPoolCode: Optional[str] = Field(None, max_length=20, min_length=1, description="股票池，限制最大长度为20，限制最小长度为1，或留空")
    period: Optional[int] = Field(None, ge=1, le=50, description="财报周期，必须大于等于1且小于等于50，或留空")
    revenueYoyType: Optional[int] = Field(None, ge=1, le=10, description="营业收入同比，必须大于等于1且小于等于10，或留空")
    nIncomeYoyType: Optional[int] = Field(None, ge=1, le=10, description="净利润同比，必须大于等于1且小于等于10，或留空")
    revenueYoyTop10: Optional[int] = Field(None, ge=1, le=10, description="营业收入行业前10%，必须大于等于1且小于等于10，或留空")
    searchKeyword: Optional[str] = Field(None, min_length=1, max_length=30, description="搜索关键词，可选，长度在1到30字符之间")

class SearchKeywordQuery(BaseModel):
    searchKeyword: Optional[str] = Field(None, min_length=1, max_length=30, description="搜索关键词，可选，长度在1到30字符之间")