"""
TO DO待完善全体数据更新逻辑
"""


import time
import requests
import pandas as pd
from fastapi import APIRouter
from src.models.financial_reports import FinancialReport   # 确保路径正确

async def combined_search_and_list():
    # 分页参数
    pageNo = 1
    pageSize = 50
    selector_url = "https://www.shidaotec.com/api/research/getFinaResearchPageList"

    # 请求第一页获取总记录数
    response = requests.get(selector_url, params={
        "pageNo": pageNo,
        "pageSize": pageSize,
        "year": "2024",
        "type": ""
    })
    response.raise_for_status()
    selector_data = response.json()

    # 获取总记录数和数据库已有记录数
    total_records = selector_data.get("data", {}).get("total", 0)
    db_record_count = await FinancialReport .all().count()
    remaining_insert_count = total_records - db_record_count

    print(f"总记录数: {total_records}，数据库已有记录数: {db_record_count}，需要插入: {remaining_insert_count}")

    # 如果已有记录数与总记录数相同，则直接结束函数
    if remaining_insert_count <= 0:
        print("数据库已包含所有记录，无需插入新数据。")
        return

    # 获取数据库中最新的记录
    latest_record = await FinancialReport .all().order_by("-date").first()
    latest_db_date = latest_record.date if latest_record else None
    latest_db_date = pd.Timestamp(latest_db_date)

    # 缓存数据
    all_data = []

    # 开始分页请求，直到遇到比数据库最新日期更早的数据
    while True:
        try:
            # 请求当前页数据
            response = requests.get(selector_url, params={
                "pageNo": pageNo,
                "pageSize": pageSize,
                "year": "2024",
                "type": ""
            })
            response.raise_for_status()
            selector_data = response.json()
            fina_data_list = selector_data.get("data", {}).get("list", [])

            # 如果页数据为空，停止循环
            if not fina_data_list:
                print("No more data available.")
                break

            # 缓存本页数据
            for record in fina_data_list:
                record.pop("seq", None)  # 移除不必要的字段
                record = {k: (None if v == "-" else v) for k, v in record.items()}
                all_data.append(record)

            # 检查本页第一条数据日期，决定是否继续请求
            first_record_date = pd.to_datetime(fina_data_list[0]["date"])
            if latest_db_date and first_record_date < latest_db_date:
                print(f"Data on page {pageNo} is older than the latest entry in the database. Stopping requests.")
                break  # 停止请求

            print(f"第 {pageNo} 页成功获取并缓存 {len(fina_data_list)} 条记录")
            pageNo += 1
            time.sleep(2)  # 请求间隔

        except requests.RequestException as e:
            print(f"Error fetching data for page {pageNo}: {e}")
            break
        except Exception as e:
            print(f"Unexpected error on page {pageNo}: {e}")
            break

    # 批量查询数据库中已存在的记录
    existing_records = await FinancialReport .filter(
        date__in=[record["date"] for record in all_data],
        period__in=[record["period"] for record in all_data],
        stock_code__in=[record["stockCode"] for record in all_data]
    ).all()

    # 将已存在记录存入集合以便于后续检查
    existing_data_set = {
        (record.date, record.period, record.stock_code, record.stock_name,
         record.industry_code, record.industry_name, record.pdf_name,
         record.pdf_url, record.revenue, record.revenue_yoy,
         record.pr_of_today, record.pr_of_after_1_day,
         record.pr_of_after_1_week, record.pr_of_after_1_month,
         record.pr_of_after_6_month, record.pr_of_that_year,
         record.nincome_yoy_lower_limit, record.nincome_yoy_upper_limit,
         record.nincome, record.nincome_yoy)
        for record in existing_records
    }

    # 开始处理缓存的数据，插入数据库
    success_count = 0
    records_to_insert = []

    for record in all_data[:]:  # 使用切片拷贝以避免在迭代时修改原始列表
        record_date = pd.to_datetime(record["date"])

        # 检查该记录是否已存在于数据库中
        if (record["date"], record["period"], record["stockCode"], record["stockName"],
            record["industryCode"], record["industryName"], record["pdfName"], record["pdfUrl"],
            record["revenue"], record["revenueYoy"], record["prOfToday"], record["prOfAfter1Day"],
            record["prOfAfter1Week"], record["prOfAfter1Month"], record["prOfAfter6Month"],
            record["prOfThatYear"], record["nincomeYoyLowerLimit"], record["nincomeYoyUpperLimit"],
            record["nincome"], record["nincomeYoy"]) in existing_data_set:
            print(f"Skipping existing record: {record}")  # 打印已跳过的记录
            continue  # 跳过已存在的记录

        # 打印即将插入的数据
        print(f"Inserting record: {record}")

        if record_date >= latest_db_date:
            # 创建待插入的 FinancialReport  对象并添加到列表
            try:
                records_to_insert.append(
                    FinancialReport (
                        date=record["date"],
                        period=record["period"],
                        year=int(record["year"]),
                        stock_code=record["stockCode"],
                        stock_name=record["stockName"],
                        industry_code=record["industryCode"] if record["industryCode"] else None,
                        industry_name=record["industryName"] if record["industryName"] else None,
                        pdf_name=record["pdfName"] if record["pdfName"] else None,
                        pdf_url=record["pdfUrl"] if record["pdfUrl"] else None,
                        revenue=float(record["revenue"]) if record["revenue"] else None,
                        revenue_yoy=float(record["revenueYoy"]) if record["revenueYoy"] else None,
                        pr_of_today=float(record["prOfToday"]) if record["prOfToday"] else None,
                        pr_of_after_1_day=float(record["prOfAfter1Day"]) if record["prOfAfter1Day"] else None,
                        pr_of_after_1_week=float(record["prOfAfter1Week"]) if record["prOfAfter1Week"] else None,
                        pr_of_after_1_month=float(record["prOfAfter1Month"]) if record["prOfAfter1Month"] else None,
                        pr_of_after_6_month=float(record["prOfAfter6Month"]) if record["prOfAfter6Month"] else None,
                        pr_of_that_year=float(record["prOfThatYear"]) if record["prOfThatYear"] else None,
                        nincome_yoy_lower_limit=record["nincomeYoyLowerLimit"] if record["nincomeYoyLowerLimit"] else None,
                        nincome_yoy_upper_limit=record["nincomeYoyUpperLimit"] if record["nincomeYoyUpperLimit"] else None,
                        nincome=float(record["nincome"]) if record["nincome"] else None,
                        nincome_yoy=float(record["nincomeYoy"]) if record["nincomeYoy"] else None
                    )
                )
                success_count += 1

            except Exception as e:
                print(f"Error processing record {record}: {e}")  # 输出处理错误

        # 成功插入所需条数后停止
        if success_count >= remaining_insert_count:
            print("this riqi:" + str(latest_db_date))
            break

    # 批量插入到数据库
    if records_to_insert:
        try:
            await FinancialReport .bulk_create(records_to_insert)
            print(f"成功插入 {len(records_to_insert)} 条记录")
        except Exception as e:
            print(f"Error during bulk insert: {e}")


    # 去重步骤
    try:
        # 查找基于 date、period 和 stock_code 的重复记录
        duplicates = await FinancialReport.raw("""
            SELECT id FROM financial_reports WHERE id NOT IN (
                SELECT id FROM (
                    SELECT MAX(id) AS id
                    FROM financial_reports
                    GROUP BY date, period, stock_code
                ) AS temp
            )
        """)
        
        duplicate_ids = [record.id for record in duplicates]
        
        if duplicate_ids:
            # 删除较旧的重复记录
            await FinancialReport.filter(id__in=duplicate_ids).delete()
            print(f"删除了 {len(duplicate_ids)} 条重复记录")
        else:
            print("未发现重复记录")
        
    except Exception as e:
        print(f"去重过程中出错: {e}")

    return {"message": "数据获取、插入和去重已完成"}